@@ -22,63 +22,84 @@ class _DataFrame:
         self.app_id = app
         self.collection_id = collection
 
-    def Read(self, original=True, itemsPerRun=100, n_jobs = 1):
-
+    def Read(self, original=True, itemsPerRun=500, n_jobs = 1):
         self.data = []
         if not Auth.tokenValid():
             Auth.refreshToken()
 
-        response = requests.get(
-            settings.baseURL + 'api/metric/' + self.app_id + '/' + self.collection_id + '?extended=true&offset=' + str(
-                0) + '&limit=' + str(itemsPerRun) + '&original=' + str(original).lower(),
+        response_total = requests.get(
+            settings.baseURL + 'api/metric/' + self.app_id + '/' + self.collection_id + '?extended=false',
             headers={
                 'Authorization': 'Bearer ' + settings.token
             })
 
-        for item in response.json()["data"]["items"]:
-            self.data.append(item["data"])
-
-        def Worker(self, i, itemsPerRun, original):
+        for i in range(0, math.ceil(response_total.json()["data"]["items"] / itemsPerRun)):
             if not Auth.tokenValid():
                 Auth.refreshToken()
             response = ""
 
+            res_data = []
             try:
+                fullUrl = settings.baseURL + 'api/metric/' + self.app_id + '/' + self.collection_id + '?extended=true&offset=' + str( i * itemsPerRun ) + '&limit=' + str(itemsPerRun) + '&original=' + str(original).lower()
                 response = requests.get(
-                    settings.baseURL + 'api/metric/' + self.app_id + '/' + self.collection_id + '?extended=true&offset=' + str(
-                    i * itemsPerRun) + '&limit=' + str(itemsPerRun) + '&original=' + str(original).lower(),
+                    fullUrl,
                     headers={
                         'Authorization': 'Bearer ' + settings.token
                 })
                 for item in response.json()["data"]["items"]:
-                    self.data.append(item["data"])
-            except:
-                print(response.status_code)
+                    res_data.append(item["data"])
 
-        if n_jobs > multiprocessing.cpu_count() or n_jobs < -1:
-            print("The maximum CPU which can be used is: {0}".format(multiprocessing.cpu_count()))
-            return
+                # Sleep for elapsed time to not get marked as DDOS
+                time.sleep(response.elapsed.total_seconds())
+                yield pd.DataFrame(res_data)
+            except requests.exceptions.RequestException as exception:
+                resp = exception.response
+                print(resp.status_code)
 
-        if n_jobs == -1:
-            n_jobs = 16
-            # n_jobs = multiprocessing.cpu_count()
 
-        if "total" in response.json().keys():
-            threadlist = []
-            for i in range(1, math.ceil(response.json()["total"] / itemsPerRun)):
-                threadlist.append(Thread(target=Worker, args=(self, i, itemsPerRun, original)))
-                if i % n_jobs == 0:
-                    for thread in threadlist:
-                        thread.start()
-                    for thread in threadlist:
-                        thread.join()
-                    threadlist = []
-            for thread in threadlist:
-                thread.start()
-            for thread in threadlist:
-                thread.join()
+        # def Worker(self, i, itemsPerRun, original):
+        #     if not Auth.tokenValid():
+        #         Auth.refreshToken()
+        #     response = ""
 
-        return pd.DataFrame(self.data)
+        #     try:
+        #         res_data = []
+        #         response = requests.get(
+        #             settings.baseURL + 'api/metric/' + self.app_id + '/' + self.collection_id + '?extended=true&offset=' + str(
+        #             i * itemsPerRun) + '&limit=' + str(itemsPerRun) + '&original=' + str(original).lower(),
+        #             headers={
+        #                 'Authorization': 'Bearer ' + settings.token
+        #         })
+        #         for item in response.json()["data"]["items"]:
+        #             res_data.append(item["data"])
+        #         yield res_data
+        #     except:
+        #         print(response.status_code)
+
+        # if n_jobs > multiprocessing.cpu_count() or n_jobs < -1:
+        #     print("The maximum CPU which can be used is: {0}".format(multiprocessing.cpu_count()))
+        #     return
+
+        # if n_jobs == -1:
+        #     n_jobs = 16
+        #     # n_jobs = multiprocessing.cpu_count()
+
+        # if "total" in response.json().keys():
+        #     threadlist = []
+        #     for i in range(1, math.ceil(response.json()["total"] / itemsPerRun)):
+        #         threadlist.append(Thread(target=Worker, args=(self, i, itemsPerRun, original)))
+        #         if i % n_jobs == 0:
+        #             for thread in threadlist:
+        #                 thread.start()
+        #             for thread in threadlist:
+        #                 thread.join()
+        #             threadlist = []
+        #     for thread in threadlist:
+        #         thread.start()
+        #     for thread in threadlist:
+        #         thread.join()
+
+        # return pd.DataFrame(self.data)
 
     def Synchronize(self, dataframe, n_jobs=1):
         if not Auth.tokenValid():
@@ -269,7 +290,8 @@ class _DataFrame:
             threadlist = []
             for x in range(0 + offset, len(dataframe.index) + offset):
                 threadlist.append(Thread(target=Worker, args=(self, amountSent, offset, dataframe, count)))
-                if i % n_jobs == 0:
+                # used to be i % n_jobs == 0: <= Did NOT work now it does
+                if n_jobs == 0:
                     for thread in threadlist:
                         thread.start()
                     for thread in threadlist:
@@ -284,7 +306,7 @@ class _DataFrame:
         else:
             raise Exception(response.json()["message"])
 
-    def Append(self, dataframe, n_jobs = 1, show = False):
+    def Append(self, dataframe, n_jobs = 1, show = False, delete_dup_columns = False):
         if not Auth.tokenValid():
             Auth.refreshToken()
         df = dataframe.copy()
@@ -360,6 +382,8 @@ class _DataFrame:
             dup_columns = dataframe.columns[dataframe.columns.duplicated()]
             if not dup_columns.any():
                 pass
+            elif delete_dup_columns is True:
+                dataframe = dataframe.loc[:,~dataframe.columns.duplicated()].copy()
             else:
                 raise TypeError
         except TypeError as error:
@@ -373,10 +397,12 @@ class _DataFrame:
         month = r'((1[0-2]){1}|(0?[1-9]){1}){1}'
         year = r'([12]{1}[0-9]{3}){1}'
         hms = r'(([2][0-3]){1}|([0-1][0-9]){1}){1}(:[0-5]{1}[0-9]{1}){2}'
+        timezone = r'\+00:00'
 
         date_dict = {
             r'\b(' + year + '-{1}' + month + '-{1}' + day + ' ' + hms + r')\b': '%Y-%m-%d %H:%M:%S',
             r'\b(' + year + '-{1}' + day + '-{1}' + month + ' ' + hms + r')\b': '%Y-%m-%d %H:%M:%S',
+            r'\b(' + year + '-{1}' + month + '-{1}' + day + ' ' + hms + timezone + r')\b': '%Y-%m-%d %H:%M:%S%z',
             r'\b(' + day + '/{1}' + month + '/{1}' + year + r')\b': '%d/%m/%Y',
             r'\b(' + month + '/{1}' + day + '/{1}' + year + r')\b': '%m/%d/%Y',
             r'\b(' + year + '/{1}' + month + '/{1}' + day + r')\b': '%Y/%m/%d',
@@ -419,7 +445,7 @@ class _DataFrame:
         for i in dates:
             for k in date_dict.keys():
                 dataframe[i] = dataframe[i].apply(
-                    lambda x: time.mktime(datetime.strptime(x, date_dict[k]).timetuple()) if type(x) == str and (
+                    lambda x: time.mktime(datetime.strptime(x[:19], date_dict[k]).timetuple()) if type(x) == str and (
                         re.match(k, x, flags=re.IGNORECASE)) else x)
 
         if show == True:
@@ -576,13 +602,13 @@ class _DataFrame:
     def DeleteItems(self, ItemIDArray=[]):
         if not Auth.tokenValid():
             Auth.refreshToken()
-        
+
         if len(ItemIDArray) > 0:
 
             x = {
                 "data": ItemIDArray
-            }        
-                    
+            }
+
             itemIDString = json.dumps(x)
 
             response = requests.delete(settings.baseURL + "api/item/" + self.app_id + "/" + self.collection_id +"/dataframe",
